==> reverie_dreams-0.4.3%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/mixin/entity/ExperienceOrbEntityMixin.java <==
package cc.thonly.reverie_dreams.mixin.entity;

import cc.thonly.reverie_dreams.entity.npc.NPCRoleEntityImpl;
import cc.thonly.reverie_dreams.interfaces.IExperienceOrbEntity;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1303;
import net.minecraft.class_1657;
import net.minecraft.class_1937;
import net.minecraft.class_243;
import net.minecraft.class_3417;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1303.class)
public abstract class ExperienceOrbEntityMixin extends class_1297 implements IExperienceOrbEntity {
    @Shadow
    @Nullable
    private class_1657 target;

    @Shadow
    public abstract int getValue();

    @Unique
    private NPCRoleEntityImpl npcTarget;

    public ExperienceOrbEntityMixin(class_1299<?> type, class_1937 world) {
        super(type, world);
    }

    @Unique
    public void moveTowardsTarget(CallbackInfo ci) {
        if (this.npcTarget == null) {
            return;
        }
        class_243 vec = new class_243(
                this.npcTarget.method_23317() - this.method_23317(),
                this.npcTarget.method_23318() + this.npcTarget.method_5751() / 2.0 - this.method_23318(),
                this.npcTarget.method_23321() - this.method_23321()
        );
        double dist2 = vec.method_1027();
        double speedFactor = 1.0 - Math.sqrt(dist2) / 8.0;
        if (speedFactor > 0) {
            this.method_18799(this.method_18798().method_1019(vec.method_1029().method_1021(speedFactor * speedFactor * 0.1)));
        }
        ci.cancel();
    }

    @Inject(method = "tick", at = @At("HEAD"))
    public void tickInject(CallbackInfo ci) {
        if (this.npcTarget == null) {
            return;
        }
        if (this.npcTarget.method_29504()) {
            return;
        }
        if (this.npcTarget.method_6032() <= 0) {
            return;
        }
        double dist2 = this.npcTarget.method_5739(this);
        if (dist2 < 1.2) {
            this.method_5783(class_3417.field_14627, 1.0f, 1.0f);
            this.npcTarget.addExperience(this.getValue());
            this.method_5650(class_5529.field_26999);
        }

    }

    @Unique
    public void setNPCTarget(NPCRoleEntityImpl npc) {
        this.npcTarget = npc;
        this.target = null;
    }

    @Override
    public NPCRoleEntityImpl getNPCTarget() {
        return this.npcTarget;
    }

    @Inject(method = "moveTowardsPlayer", at = @At("HEAD"), cancellable = true)
    public void moveToPlayerTickBefore(CallbackInfo ci) {
        this.moveTowardsTarget(ci);
    }
}

==> reverie_dreams-0.4.7%2B1.21.7%2Bbeta-sources.jar: cc/thonly/reverie_dreams/mixin/entity/ExperienceOrbEntityMixin.java <==
package cc.thonly.reverie_dreams.mixin.entity;

import cc.thonly.reverie_dreams.entity.npc.NPCRoleEntity;
import cc.thonly.reverie_dreams.interfaces.IExperienceOrbEntity;
import net.minecraft.class_1297;
import net.minecraft.class_1299;
import net.minecraft.class_1303;
import net.minecraft.class_1657;
import net.minecraft.class_1937;
import net.minecraft.class_243;
import net.minecraft.class_3417;
import org.jetbrains.annotations.Nullable;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_1303.class)
public abstract class ExperienceOrbEntityMixin extends class_1297 implements IExperienceOrbEntity {
    @Shadow
    @Nullable
    private class_1657 followingPlayer;

    @Shadow
    public abstract int getValue();

    @Unique
    private NPCRoleEntity npcTarget;

    public ExperienceOrbEntityMixin(class_1299<?> type, class_1937 world) {
        super(type, world);
    }

    @Unique
    public void moveTowardsTarget(CallbackInfo ci) {
        if (this.npcTarget == null) {
            return;
        }
        class_243 vec = new class_243(
                this.npcTarget.method_23317() - this.method_23317(),
                this.npcTarget.method_23318() + this.npcTarget.method_5751() / 2.0 - this.method_23318(),
                this.npcTarget.method_23321() - this.method_23321()
        );
        double dist2 = vec.method_1027();
        double speedFactor = 1.0 - Math.sqrt(dist2) / 8.0;
        if (speedFactor > 0) {
            this.method_18799(this.method_18798().method_1019(vec.method_1029().method_1021(speedFactor * speedFactor * 0.1)));
        }
        ci.cancel();
    }

    @Inject(method = "tick", at = @At("HEAD"))
    public void tickInject(CallbackInfo ci) {
        if (this.npcTarget == null) {
            return;
        }
        if (this.npcTarget.method_29504()) {
            return;
        }
        if (this.npcTarget.method_6032() <= 0) {
            return;
        }
        double dist2 = this.npcTarget.method_5739(this);
        if (dist2 < 1.2) {
            this.method_5783(class_3417.field_14627, 1.0f, 1.0f);
            this.npcTarget.addExperience(this.getValue());
            this.method_5650(class_5529.field_26999);
        }

    }

    @Unique
    public void setNPCTarget(NPCRoleEntity npc) {
        this.npcTarget = npc;
        this.followingPlayer = null;
    }

    @Override
    public NPCRoleEntity getNPCTarget() {
        return this.npcTarget;
    }

    @Inject(method = "followNearbyPlayer", at = @At("HEAD"), cancellable = true)
    public void moveToPlayerTickBefore(CallbackInfo ci) {
        this.moveTowardsTarget(ci);
    }
}
